// Flying Flag Shader by Satoshi Ueyama

float4x4 matWorldViewProj;
float3 vecLight;
float fly_time;
float fly_scale;
float tustep;
float tvstep;
texture Tex0;

float4x4 matWorldView;

sampler Sampler = sampler_state
{
    Texture   = (Tex0);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

struct VS_INPUT
{
    float3 pos  : POSITION0;
    float3 posN1 : POSITION1;
    float3 posN2 : POSITION2;
    float4 diffuse : COLOR0;
    float2 tex  : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 pos  : POSITION;
    float4 diffuse : COLOR0;
    float2 tex  : TEXCOORD0;
};

VS_OUTPUT FlagVS(VS_INPUT In)
{
	VS_OUTPUT Out = (VS_OUTPUT)0;

	In.pos.z   += In.tex.x*            sin(In.tex.x            * 8 + fly_time) * cos(In.tex.y            * 6 + fly_time) * fly_scale;
	In.posN1.z += (In.tex.x - tustep)* sin((In.tex.x - tustep) * 8 + fly_time) * cos(In.tex.y            * 6 + fly_time) * fly_scale;
	In.posN2.z += In.tex.x*            sin(In.tex.x            * 8 + fly_time) * cos((In.tex.y - tvstep) * 6 + fly_time) * fly_scale;

	In.pos.y   += sin(In.tex.x            * 8 + fly_time*2) * fly_scale/4;
	In.posN1.y += sin((In.tex.x - tustep) * 8 + fly_time*2) * fly_scale/4;
	In.posN2.y += sin(In.tex.x            * 8 + fly_time*2) * fly_scale/4;

    float3 L = -vecLight;

	float3 E1 = In.posN1 - In.pos;
	float3 E2 = In.posN2 - In.pos;
	
	//cross product
	float3 N;
	N.x = E1.y * E2.z - E1.z * E2.y;
	N.y = E1.z * E2.x - E1.x * E2.z;
	N.z = E1.x * E2.y - E1.y * E2.x;

	Out.pos  = mul(float4(In.pos, 1), matWorldViewProj);
	Out.diffuse = In.diffuse * (max(0, dot(normalize( mul(N, (float3x3)matWorldView) ), L)) + 0.36f );
	Out.tex  = In.tex;         

	return Out;
}

technique FlagShader
{
	pass P0
	{
		Sampler[0] = (Sampler);

		AlphaBlendEnable = FALSE;
		ZEnable = 1;
		ZWriteEnable = TRUE;

		VertexShader = compile vs_1_1 FlagVS();
		PixelShader = null;
	}
}